/*	UEFTapeFileReader.java 1.20 Release Candidate
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	(C) 2004 Richard Brown
 *
 */

package com.rwbrown.audio.spi.uef;

import java.util.zip.GZIPInputStream;

import java.io.InputStream;
import java.io.FileInputStream;
import java.io.BufferedInputStream;
import java.io.File;
import java.io.IOException;

import java.net.URL;

import javax.sound.sampled.spi.AudioFileReader;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.AudioFileFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.UnsupportedAudioFileException;

public class UEFTapeFileReader extends AudioFileReader {
	
	private static class UEFTapeFileFormatType 
		extends AudioFileFormat.Type {
		private static final AudioFileFormat.Type UEF	= 
			new UEFTapeFileFormatType("UEF","uef");
		public UEFTapeFileFormatType(String name, String extension) {
			super(name,extension);
		}
	}
		
	private static final AudioFileFormat UEF_AUDIO_FILE_FORMAT	=
		new AudioFileFormat(UEFTapeFileFormatType.UEF,
			UEFTapePCMInputStream.AUDIOFORMAT, AudioSystem.NOT_SPECIFIED);
	
	
	private static UEFTapePCMInputStream getUEFStream(
		InputStream source, boolean forUse) throws IOException {
		if (!source.markSupported()) 
			throw new IOException("Unbuffered InputStream");
		source.mark(1024);
		InputStream target	= source;
		int magic			= 0;
		try {
			magic			= source.read() | source.read()<<8;
			source.reset();
			if (magic==GZIPInputStream.GZIP_MAGIC)
				target		= new BufferedInputStream(
					new GZIPInputStream(source));
			return new UEFTapePCMInputStream(target);
		} catch (IOException e) {
			source.reset();
			throw e;
		} finally {
			if (!forUse) source.reset();
		}
	}
	
	public AudioFileFormat getAudioFileFormat(InputStream in)
		throws UnsupportedAudioFileException, IOException {
		getUEFStream(in, false);
		return UEF_AUDIO_FILE_FORMAT;
	}
	
	public AudioFileFormat getAudioFileFormat(File file) 
		throws UnsupportedAudioFileException, IOException {
		InputStream in	= new BufferedInputStream(
			new FileInputStream(file));
		try {
			return getAudioFileFormat(in);
		} finally {
			in.close();
		}
	}
	
	public AudioFileFormat getAudioFileFormat(URL url) 
		throws UnsupportedAudioFileException, IOException {
		InputStream in	= new BufferedInputStream(
			url.openStream());
		try {
			return getAudioFileFormat(in);
		} finally {
			in.close();
		}		
	}
	
	public AudioInputStream getAudioInputStream(InputStream in)
		throws UnsupportedAudioFileException, IOException {
		return new AudioInputStream(getUEFStream(in, true), 
			UEF_AUDIO_FILE_FORMAT.getFormat(),
			AudioSystem.NOT_SPECIFIED);
	}
	
	public AudioInputStream getAudioInputStream(File file) 
		throws UnsupportedAudioFileException, IOException {
		InputStream in	= new BufferedInputStream(
			new FileInputStream(file));
		try {
			return getAudioInputStream(in);
		} catch (UnsupportedAudioFileException e) {
			in.close();
			throw e;
		} catch (IOException e) {
			in.close();
			throw e;
		}
	}
	
	public AudioInputStream getAudioInputStream(URL url) 
		throws UnsupportedAudioFileException, IOException {
		InputStream in	= new BufferedInputStream(
			url.openStream());
		try {
			return getAudioInputStream(in);
		} catch (UnsupportedAudioFileException e) {
			in.close();
			throw e;
		} catch (IOException e) {
			in.close();
			throw e;
		}
	}	
}